#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

# NOTE: this is a special build of root v5 requested by the GENIE development team

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e9|e9:nu|e7|e7:nu|e10|e10:nu|i1|i1:nu> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=root
origpkgver=v5_34_36
pkgver=${origpkgver}
ssibuildshims_version=v1_00_00
pkgtarver=`echo ${origpkgver} | sed -e 's/_/./g'`
pkgtarfile=root_${pkgtarver}.source.tar.gz

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

    if [ "${basequal}" = "e9" ]
    then
	shortqual=e
	cqual=e9
        xcc=gcc
    elif [ "${basequal}" = "e9:nu" ]
    then
	shortqual=nu
	cqual=e9
        xcc=gcc
    elif [ "${basequal}" = "e7" ]
    then
	shortqual=e
	cqual=e7
        xcc=gcc
    elif [ "${basequal}" = "e7:nu" ]
    then
	shortqual=nu
	cqual=e7
        xcc=gcc
    elif [ "${basequal}" = "e10" ]
    then
	shortqual=e
	cqual=e10
        xcc=gcc
    elif [ "${basequal}" = "e10:nu" ]
    then
	shortqual=nu
	cqual=e10
        xcc=gcc
    elif [ "${basequal}" = "i1" ]
    then
	shortqual=i
	cqual=i1
        xcc=icc
        (( useicc = 1 ))
    elif [ "${basequal}" = "i1:nu" ]
    then
	shortqual=nu
	cqual=i1
        xcc=icc
        (( useicc = 1 ))
    else
       echo "ERROR: unrecognized qualifier"
       usage
       exit 1
    fi

# use ROOTSYS/MyRules.mk
if [ "${extraqual}" = "opt" ]
then
    rules=MyRules_opt.mk
elif [ "${extraqual}" = "debug" ]
then
    extracommand="--build=debug"
    rules=MyRules_debug.mk
elif [ "${extraqual}" = "prof" ]
then
    rules=MyRules_prof.mk
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup root
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

if [ -z ${FFTW_INC} ]
then
   echo "ERROR: failed to setup fftw"
   exit 1
else
   echo "FFTW_INC: ${FFTW_INC}"
fi

if [ -z ${LIBXML2_INC} ]
then
   echo "ERROR: failed to setup libxml"
   exit 1
else
   echo "LIBXML2_INC: ${LIBXML2_INC}"
fi

if [ -z ${XROOTD_INC} ]
then
echo "ERROR: failed to setup xrootd"
exit 1
else
echo "XROOTD_INC: ${XROOTD_INC}"
fi

if [ "${shortqual}" = "nu" ]
then
    if [ -z ${PYTHIA_VERSION} ]
    then
       echo "ERROR: failed to setup postgresql"
       exit 1
    else
       echo "postgresql version: ${PYTHIA_VERSION}"
    fi
    if [ -z ${MYSQL_CLIENT_VERSION} ]
    then
       echo "ERROR: failed to setup mysql_client"
       exit 1
    else
       echo "mysql_client version: ${MYSQL_CLIENT_VERSION}"
    fi
    if [ -z ${PYTHIA_VERSION} ]
    then
       echo "ERROR: failed to setup pythia"
       exit 1
    else
       echo "pythia version: ${PYTHIA_VERSION}"
    fi
fi

if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
fi

cd ${sourcedir} && rm -rf root
tar xf ${tardir}/${pkgtarfile} || ssi_die "Unable to untar source"

# get the extra rules 
if [ -n ${rules} ]
then
   cp ${patchdir}/${rules} ${blddir}/MyRules.mk
fi
if [[ "${cqual}" == e[679] ]]
then
  c11="--enable-cxx14"
  compiler_command="--with-cc=gcc --with-cxx=g++ --with-ld=g++"
elif [[ "${cqual}" == e10 ]]
then
  c11="--enable-cxx14"
  compiler_command="--with-cc=gcc --with-cxx=g++ --with-ld=g++"
elif [ "${cqual}" = "i1" ]
then
  c11="--enable-c++11"
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}/root

# apply Fermi patches
for patch in ${patchdir}/fermi_root_5_34.patch; do
  patch ${DEBUG_PATCHES:+--verbose} -p0 < "${patch}" || ssi_die "ERROR: application of patch ${patch} failed."
done

[[ -n "${DEBUG_PATCHES}" ]] && exit 1

cd ${blddir}

echo "Building with gcc: $(which gcc) and g++: $(which g++)" 1>&2

if [ ${OS1} = "Darwin" ]
then
  darwin_xopts=(--disable-cocoa --enable-builtin-afterimage \
                --enable-builtin-ftgl --enable-builtin-freetype \
                --enable-builtin-glew --enable-builtin-pcre \
                --enable-builtin-zlib --enable-builtin-lzma \
                --disable-bonjour \
                --cxxflags="-Wno-deprecated-declarations")
  makearch="macosx64${useicc:+icc}"
  # Remove /usr/local from search paths.
  perl -wapi\.bak -e 'm&prefix& or (s&/usr/local[^\s;"'"'"'\)]*&&g);' ${sourcedir}/root/configure
else
  makearch="linuxx8664${xcc}"
fi

if [ "${shortqual}" = "nu" ]
then
${sourcedir}/root/configure ${makearch} ${extracommand} ${compiler_command} \
--enable-http \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--with-xrootd-incdir=${XROOTD_INC}/xrootd \
--with-xrootd-libdir=${XROOTDROOT}/lib/ \
--enable-minuit2 \
--disable-xft \
--with-gsl-incdir=${GSL_INC} \
--with-gsl-libdir=${GSL_LIB} \
--disable-sqlite \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib \
--with-pythia6-libdir=${PYLIB} \
--with-pgsql-incdir=${POSTGRESQL_FQ_DIR}/include \
--with-pgsql-libdir=${POSTGRESQL_FQ_DIR}/lib \
--with-mysql-incdir=${MYSQL_CLIENT_FQ_DIR}/include \
--with-mysql-libdir=${MYSQL_CLIENT_FQ_DIR}/lib \
--enable-mathmore \
--enable-gsl-shared \
${darwin:+"${darwin_xopts[@]}"}
else # Vanilla now has minuit2
${sourcedir}/root/configure ${makearch} ${extracommand}  ${compiler_command} \
--enable-http \
--enable-gdml ${c11} \
--enable-roofit \
--with-fftw3-incdir=${FFTW_INC} \
--with-fftw3-libdir=${FFTW_FQ_DIR}/lib/ \
--disable-mysql \
--with-xrootd-incdir=${XROOTD_INC}/xrootd \
--with-xrootd-libdir=${XROOTDROOT}/lib/ \
--enable-minuit2 \
--disable-xft \
--disable-sqlite \
--with-xml-incdir=${LIBXML2_INC} \
--with-xml-libdir=${LIBXML2_FQ_DIR}/lib \
${darwin:+"${darwin_xopts[@]}"}
fi

(( $? == 0 )) || ssi_die "ERROR: configure failed."

make -j ${ncore} || ssi_die "ERROR: make failed."

# now install in ROOTSYS
make install || ssi_die "ERROR: make install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# need to fix paths:
# bin/root-config

fixfile=/tmp/root-fix-${qualdir}-$$$$.sh.sh
rm -f $fixfile

echo "#!/bin/bash" > $fixfile
##echo " set -x " >>  $fixfile
echo "mv ${pkgdir}/bin/root-config ${pkgdir}/bin/root-config.bak" >> $fixfile
echo "cat ${pkgdir}/bin/root-config.bak | \\" >> $fixfile
echo "     sed -e 's%${LIBXML2_FQ_DIR}%\${LIBXML2_FQ_DIR}%g' \\" >> $fixfile
echo "         -e 's%${FFTW_FQ_DIR}%\${FFTW_FQ_DIR}%g' \\" >> $fixfile
echo "         -e 's%${XROOTDROOT}%\${XROOTDROOT}%g' \\" >> $fixfile
if [ "${shortqual}" = "nu" ]
then
echo "         -e 's%${PYLIB}%\${PYLIB}%g' \\" >> $fixfile
echo "         -e 's%${POSTGRESQL_FQ_DIR}%\${POSTGRESQL_FQ_DIR}%g' \\" >> $fixfile
echo "         -e 's%${MYSQL_CLIENT_FQ_DIR}%\${MYSQL_CLIENT_FQ_DIR}%g' \\" >> $fixfile
fi
echo "     > ${pkgdir}/bin/root-config || exit 1" >> $fixfile
echo "chmod +x  ${pkgdir}/bin/root-config" >> $fixfile
echo "exit \$?" >> $fixfile
chmod +x $fixfile
$fixfile || ssi_die "Execution of fixfile script was unsuccessful."

rm -f $fixfile

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup failed"
echo "${package} is installed at ${ROOTSYS}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
